package io.github.beradq.adybubbles

import ink.ptms.adyeshach.core.entity.EntityInstance
import ink.ptms.adyeshach.core.event.AdyeshachEntityVisibleEvent
import ink.ptms.adyeshach.impl.entity.trait.Trait
import ink.ptms.adyeshach.impl.util.Inputs.inputBook
import taboolib.common.LifeCycle
import org.bukkit.entity.Player
import taboolib.common.platform.Awake
import taboolib.common.platform.event.EventPriority
import taboolib.common.platform.event.SubscribeEvent
import taboolib.common.platform.function.submit
import taboolib.common.platform.service.PlatformExecutor
import taboolib.module.chat.uncolored
import java.util.concurrent.CompletableFuture

object TraitBubblesChat : Trait() {
    private val bubbleLines = mutableMapOf<String, List<String>>()
    private val bubbleState = mutableMapOf<String, Int>()
    private val visibleEntity = mutableListOf<String>()
    private var handle: PlatformExecutor.PlatformTask? = null

    override fun edit(player: Player, entityInstance: EntityInstance): CompletableFuture<Void> {
        val future = CompletableFuture<Void>()
        val content = bubbleLines.getOrPut(entityInstance.uniqueId) { mutableListOf() }
        player.inputBook(content) {
            future.complete(null)
            if (it.all { s -> s.isBlank() }) {
                data[entityInstance.uniqueId] = null
                bubbleState.remove(entityInstance.uniqueId)
                bubbleLines.remove(entityInstance.uniqueId)
                BubblesBundle.clear(entityInstance.uniqueId)
            } else {
                data[entityInstance.uniqueId] = it.uncolored()
            }
            update(entityInstance.uniqueId)
        }
        return future
    }

    private fun update(entityInstanceUniqueId: String) {
        if (!data.contains(entityInstanceUniqueId)) return
        val lines = data.getStringList(entityInstanceUniqueId)
        bubbleLines[entityInstanceUniqueId] = lines
    }

    private fun fistUpdate(entityInstanceUniqueId: String) {
        if (bubbleLines.containsKey(entityInstanceUniqueId)) return
        update(entityInstanceUniqueId)
    }


    @Awake(LifeCycle.ACTIVE)
    private fun startTicking() {
        handle?.cancel()
        handle = submit(period = Config.bubblesConfig.chat.period, async = true) {
            val ve = visibleEntity.toList()
            ve.forEach {
                nextBubble(it)
            }
        }
    }

    @SubscribeEvent(priority = EventPriority.MONITOR, ignoreCancelled = true)
    private fun onVisible(e: AdyeshachEntityVisibleEvent) {
        if (e.entity.isPublic()) {
            fistUpdate(e.entity.uniqueId)
            if (e.visible) {
                if (visibleEntity.contains(e.entity.uniqueId)) return
                visibleEntity.add(e.entity.uniqueId)
            } else {
                visibleEntity.remove(e.entity.uniqueId)
                BubblesBundle.clear(e.entity.uniqueId)
            }
        }
    }

    private fun nextBubble(entityInstanceUniqueId: String) {
        val bubbleLine = bubbleLines[entityInstanceUniqueId] ?: return
        if (bubbleLine.isEmpty()) return
        var state = bubbleState[entityInstanceUniqueId] ?: 0
        if (state >= bubbleLine.size) {
            state = 0
        }
        val currentLine = bubbleLine[state]
        BubblesBundle.popup(entityInstanceUniqueId, currentLine)
        BubblesBundle.limit(entityInstanceUniqueId, Config.bubblesConfig.chat.limit)
        bubbleState[entityInstanceUniqueId] = state
    }

    override fun id(): String {
        return "bubbles-chat"
    }
}